@@ -1,7 +1,7 @@
 /*
  *  handle.c
  *
- *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2010 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *  Copyright (c) 2005 by Aurelien Foret <orelien@chez.com>
  *  Copyright (c) 2005, 2006 by Miklos Vajna <vmiklos@frugalware.org>
@@ -79,6 +79,7 @@ void _alpm_handle_free(pmhandle_t *handle)
 	FREELIST(handle->cachedirs);
 	FREE(handle->logfile);
 	FREE(handle->lockfile);
+	FREE(handle->arch);
 	FREELIST(handle->dbs_sync);
 	FREELIST(handle->noupgrade);
 	FREELIST(handle->noextract);
@@ -168,7 +169,7 @@ const char SYMEXPORT *alpm_option_get_lockfile()
 	return handle->lockfile;
 }
 
-unsigned short SYMEXPORT alpm_option_get_usesyslog()
+int SYMEXPORT alpm_option_get_usesyslog()
 {
 	if (handle == NULL) {
 		pm_errno = PM_ERR_HANDLE_NULL;
@@ -213,13 +214,22 @@ alpm_list_t SYMEXPORT *alpm_option_get_ignoregrps()
 	return handle->ignoregrp;
 }
 
-unsigned short SYMEXPORT alpm_option_get_nopassiveftp()
+const char SYMEXPORT *alpm_option_get_arch()
+{
+	if (handle == NULL) {
+		pm_errno = PM_ERR_HANDLE_NULL;
+		return NULL;
+	}
+	return handle->arch;
+}
+
+int SYMEXPORT alpm_option_get_usedelta()
 {
 	if (handle == NULL) {
 		pm_errno = PM_ERR_HANDLE_NULL;
 		return -1;
 	}
-	return handle->nopassiveftp;
+	return handle->usedelta;
 }
 
 pmdb_t SYMEXPORT *alpm_option_get_localdb()
@@ -436,7 +446,7 @@ int SYMEXPORT alpm_option_set_logfile(const char *logfile)
 	return(0);
 }
 
-void SYMEXPORT alpm_option_set_usesyslog(unsigned short usesyslog)
+void SYMEXPORT alpm_option_set_usesyslog(int usesyslog)
 {
 	handle->usesyslog = usesyslog;
 }
@@ -529,12 +539,13 @@ int SYMEXPORT alpm_option_remove_ignoregrp(const char *grp)
 	return(0);
 }
 
-void SYMEXPORT alpm_option_set_nopassiveftp(unsigned short nopasv)
+void SYMEXPORT alpm_option_set_arch(const char *arch)
 {
-	handle->nopassiveftp = nopasv;
+	if(handle->arch) FREE(handle->arch);
+	if(arch) handle->arch = strdup(arch);
 }
 
-void SYMEXPORT alpm_option_set_usedelta(unsigned short usedelta)
+void SYMEXPORT alpm_option_set_usedelta(int usedelta)
 {
 	handle->usedelta = usedelta;
 }
