@@ -1,7 +1,7 @@
 /*
  *  handle.h
  *
- *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2010 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -57,9 +57,9 @@ typedef struct _pmhandle_t {
 	alpm_list_t *ignoregrp;   /* List of groups to ignore */
 
 	/* options */
-	unsigned short usesyslog;    /* Use syslog instead of logfile? */ /* TODO move to frontend */
-	unsigned short nopassiveftp; /* Don't use PASV ftp connections */
-	unsigned short usedelta;     /* Download deltas if possible */
+	int usesyslog;    /* Use syslog instead of logfile? */ /* TODO move to frontend */
+	char *arch;       /* Architecture of packages we should allow */
+	int usedelta;     /* Download deltas if possible */
 } pmhandle_t;
 
 /* global handle variable */
