@@ -1,7 +1,7 @@
 /*
  *  package.c
  *
- *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2010 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *  Copyright (c) 2005 by Aurelien Foret <orelien@chez.com>
  *  Copyright (c) 2005, 2006 by Christian Hamar <krics@linuxforum.hu>
@@ -295,7 +295,7 @@ alpm_list_t SYMEXPORT *alpm_pkg_get_groups(pmpkg_t *pkg)
 	return pkg->groups;
 }
 
-unsigned short SYMEXPORT alpm_pkg_has_force(pmpkg_t *pkg)
+int SYMEXPORT alpm_pkg_has_force(pmpkg_t *pkg)
 {
 	ALPM_LOG_FUNC;
 
@@ -423,13 +423,6 @@ alpm_list_t SYMEXPORT *alpm_pkg_get_backup(pmpkg_t *pkg)
 	return pkg->backup;
 }
 
-alpm_list_t SYMEXPORT *alpm_pkg_get_removes(pmpkg_t *pkg)
-{
-	ASSERT(pkg != NULL, return(NULL));
-
-	return(pkg->removes);
-}
-
 pmdb_t SYMEXPORT *alpm_pkg_get_db(pmpkg_t *pkg)
 {
 	/* Sanity checks */
@@ -466,7 +459,6 @@ void SYMEXPORT *alpm_pkg_changelog_open(pmpkg_t *pkg)
 		struct archive *archive = NULL;
 		struct archive_entry *entry;
 		const char *pkgfile = pkg->origin_data.file;
-		int ret = ARCHIVE_OK;
 
 		if((archive = archive_read_new()) == NULL) {
 			RET_ERR(PM_ERR_LIBARCHIVE, NULL);
@@ -480,7 +472,7 @@ void SYMEXPORT *alpm_pkg_changelog_open(pmpkg_t *pkg)
 			RET_ERR(PM_ERR_PKG_OPEN, NULL);
 		}
 
-		while((ret = archive_read_next_header(archive, &entry)) == ARCHIVE_OK) {
+		while(archive_read_next_header(archive, &entry) == ARCHIVE_OK) {
 			const char *entry_name = archive_entry_pathname(entry);
 
 			if(strcmp(entry_name, ".CHANGELOG") == 0) {
@@ -548,7 +540,7 @@ int SYMEXPORT alpm_pkg_changelog_close(const pmpkg_t *pkg, void *fp)
 	return(ret);
 }
 
-unsigned short SYMEXPORT alpm_pkg_has_scriptlet(pmpkg_t *pkg)
+int SYMEXPORT alpm_pkg_has_scriptlet(pmpkg_t *pkg)
 {
 	ALPM_LOG_FUNC;
 
@@ -563,6 +555,23 @@ unsigned short SYMEXPORT alpm_pkg_has_scriptlet(pmpkg_t *pkg)
 	return pkg->scriptlet;
 }
 
+static void find_requiredby(pmpkg_t *pkg, pmdb_t *db, alpm_list_t **reqs)
+{
+	const alpm_list_t *i;
+	for(i = _alpm_db_get_pkgcache(db); i; i = i->next) {
+		if(!i->data) {
+			continue;
+		}
+		pmpkg_t *cachepkg = i->data;
+		if(_alpm_dep_edge(cachepkg, pkg)) {
+			const char *cachepkgname = cachepkg->name;
+			if(alpm_list_find_str(*reqs, cachepkgname) == 0) {
+				*reqs = alpm_list_add(*reqs, strdup(cachepkgname));
+			}
+		}
+	}
+}
+
 /**
  * @brief Compute the packages requiring a given package.
  * @param pkg a package
@@ -572,192 +581,29 @@ alpm_list_t SYMEXPORT *alpm_pkg_compute_requiredby(pmpkg_t *pkg)
 {
 	const alpm_list_t *i;
 	alpm_list_t *reqs = NULL;
+	pmdb_t *db;
 
-	pmdb_t *localdb = alpm_option_get_localdb();
-	for(i = _alpm_db_get_pkgcache(localdb); i; i = i->next) {
-		if(!i->data) {
-			continue;
-		}
-		pmpkg_t *cachepkg = i->data;
-		if(_alpm_dep_edge(cachepkg, pkg)) {
-			const char *cachepkgname = alpm_pkg_get_name(cachepkg);
-			reqs = alpm_list_add(reqs, strdup(cachepkgname));
+	if(pkg->origin == PKG_FROM_FILE) {
+		/* The sane option; search locally for things that require this. */
+		db = alpm_option_get_localdb();
+		find_requiredby(pkg, db, &reqs);
+	} else {
+		/* We have a DB package. if it is a local package, then we should
+		 * only search the local DB; else search all known sync databases. */
+		db = pkg->origin_data.db;
+		if(db->is_local) {
+			find_requiredby(pkg, db, &reqs);
+		} else {
+			for(i = handle->dbs_sync; i; i = i->next) {
+				db = i->data;
+				find_requiredby(pkg, db, &reqs);
+				reqs = alpm_list_msort(reqs, alpm_list_count(reqs), _alpm_str_cmp);
+			}
 		}
 	}
 	return(reqs);
 }
 
-/** Compare two version strings and determine which one is 'newer'.
- * Returns a value comparable to the way strcmp works. Returns 1
- * if a is newer than b, 0 if a and b are the same version, or -1
- * if b is newer than a.
- *
- * This function has been adopted from the rpmvercmp function located
- * at lib/rpmvercmp.c, and was most recently updated against rpm
- * version 4.4.2.3. Small modifications have been made to make it more
- * consistent with the libalpm coding style.
- *
- * Keep in mind that the pkgrel is only compared if it is available
- * on both versions handed to this function. For example, comparing
- * 1.5-1 and 1.5 will yield 0; comparing 1.5-1 and 1.5-2 will yield
- * -1 as expected. This is mainly for supporting versioned dependencies
- * that do not include the pkgrel.
- */
-int SYMEXPORT alpm_pkg_vercmp(const char *a, const char *b)
-{
-	char oldch1, oldch2;
-	char *str1, *str2;
-	char *ptr1, *ptr2;
-	char *one, *two;
-	int rc;
-	int isnum;
-	int ret = 0;
-
-	ALPM_LOG_FUNC;
-
-	/* libalpm added code. ensure our strings are not null */
-	if(!a) {
-		if(!b) return(0);
-		return(-1);
-	}
-	if(!b) return(1);
-
-	/* easy comparison to see if versions are identical */
-	if(strcmp(a, b) == 0) return(0);
-
-	str1 = strdup(a);
-	str2 = strdup(b);
-
-	one = str1;
-	two = str2;
-
-	/* loop through each version segment of str1 and str2 and compare them */
-	while(*one && *two) {
-		while(*one && !isalnum((int)*one)) one++;
-		while(*two && !isalnum((int)*two)) two++;
-
-		/* If we ran to the end of either, we are finished with the loop */
-		if(!(*one && *two)) break;
-
-		ptr1 = one;
-		ptr2 = two;
-
-		/* grab first completely alpha or completely numeric segment */
-		/* leave one and two pointing to the start of the alpha or numeric */
-		/* segment and walk ptr1 and ptr2 to end of segment */
-		if(isdigit((int)*ptr1)) {
-			while(*ptr1 && isdigit((int)*ptr1)) ptr1++;
-			while(*ptr2 && isdigit((int)*ptr2)) ptr2++;
-			isnum = 1;
-		} else {
-			while(*ptr1 && isalpha((int)*ptr1)) ptr1++;
-			while(*ptr2 && isalpha((int)*ptr2)) ptr2++;
-			isnum = 0;
-		}
-
-		/* save character at the end of the alpha or numeric segment */
-		/* so that they can be restored after the comparison */
-		oldch1 = *ptr1;
-		*ptr1 = '\0';
-		oldch2 = *ptr2;
-		*ptr2 = '\0';
-
-		/* this cannot happen, as we previously tested to make sure that */
-		/* the first string has a non-null segment */
-		if (one == ptr1) {
-			ret = -1;	/* arbitrary */
-			goto cleanup;
-		}
-
-		/* take care of the case where the two version segments are */
-		/* different types: one numeric, the other alpha (i.e. empty) */
-		/* numeric segments are always newer than alpha segments */
-		/* XXX See patch #60884 (and details) from bugzilla #50977. */
-		if (two == ptr2) {
-			ret = isnum ? 1 : -1;
-			goto cleanup;
-		}
-
-		if (isnum) {
-			/* this used to be done by converting the digit segments */
-			/* to ints using atoi() - it's changed because long  */
-			/* digit segments can overflow an int - this should fix that. */
-
-			/* throw away any leading zeros - it's a number, right? */
-			while (*one == '0') one++;
-			while (*two == '0') two++;
-
-			/* whichever number has more digits wins */
-			if (strlen(one) > strlen(two)) {
-				ret = 1;
-				goto cleanup;
-			}
-			if (strlen(two) > strlen(one)) {
-				ret = -1;
-				goto cleanup;
-			}
-		}
-
-		/* strcmp will return which one is greater - even if the two */
-		/* segments are alpha or if they are numeric.  don't return  */
-		/* if they are equal because there might be more segments to */
-		/* compare */
-		rc = strcmp(one, two);
-		if (rc) {
-			ret = rc < 1 ? -1 : 1;
-			goto cleanup;
-		}
-
-		/* restore character that was replaced by null above */
-		*ptr1 = oldch1;
-		one = ptr1;
-		*ptr2 = oldch2;
-		two = ptr2;
-
-		/* libalpm added code. check if version strings have hit the pkgrel
-		 * portion. depending on which strings have hit, take correct action.
-		 * this is all based on the premise that we only have one dash in
-		 * the version string, and it separates pkgver from pkgrel. */
-		if(*ptr1 == '-' && *ptr2 == '-') {
-			/* no-op, continue comparing since we are equivalent throughout */
-		} else if(*ptr1 == '-') {
-			/* ptr1 has hit the pkgrel and ptr2 has not. continue version
-			 * comparison after stripping the pkgrel from ptr1. */
-			*ptr1 = '\0';
-		} else if(*ptr2 == '-') {
-			/* ptr2 has hit the pkgrel and ptr1 has not. continue version
-			 * comparison after stripping the pkgrel from ptr2. */
-			*ptr2 = '\0';
-		}
-	}
-
-	/* this catches the case where all numeric and alpha segments have */
-	/* compared identically but the segment separating characters were */
-	/* different */
-	if ((!*one) && (!*two)) {
-		ret = 0;
-		goto cleanup;
-	}
-
-	/* the final showdown. we never want a remaining alpha string to
-	 * beat an empty string. the logic is a bit weird, but:
-	 * - if one is empty and two is not an alpha, two is newer.
-	 * - if one is an alpha, two is newer.
-	 * - otherwise one is newer.
-	 * */
-	if ( ( !*one && !isalpha((int)*two) )
-			|| isalpha((int)*one) ) {
-		ret = -1;
-	} else {
-		ret = 1;
-	}
-
-cleanup:
-	free(str1);
-	free(str2);
-	return(ret);
-}
-
 /** @} */
 
 pmpkg_t *_alpm_pkg_new(void)
@@ -858,7 +704,11 @@ void _alpm_pkg_free(pmpkg_t *pkg)
 	FREE(pkg);
 }
 
-/* Free transaction specific fields */
+/* This function should be used when removing a target from upgrade/sync target list
+ * Case 1: If pkg is a loaded package file (PKG_FROM_FILE), it will be freed.
+ * Case 2: If pkg is a pkgcache entry (PKG_FROM_CACHE), it won't be freed,
+ *         only the transaction specific fields of pkg will be freed.
+ */
 void _alpm_pkg_free_trans(pmpkg_t *pkg)
 {
 	ALPM_LOG_FUNC;
@@ -867,6 +717,11 @@ void _alpm_pkg_free_trans(pmpkg_t *pkg)
 		return;
 	}
 
+	if(pkg->origin == PKG_FROM_FILE) {
+		_alpm_pkg_free(pkg);
+		return;
+	}
+
 	alpm_list_free(pkg->removes);
 	pkg->removes = NULL;
 }
@@ -894,7 +749,7 @@ int _alpm_pkg_cmp(const void *p1, const void *p2)
 {
 	pmpkg_t *pkg1 = (pmpkg_t *)p1;
 	pmpkg_t *pkg2 = (pmpkg_t *)p2;
-	return(strcmp(alpm_pkg_get_name(pkg1), alpm_pkg_get_name(pkg2)));
+	return(strcmp(pkg1->name, pkg2->name));
 }
 
 /* Test for existence of a package in a alpm_list_t*
@@ -913,7 +768,7 @@ pmpkg_t *_alpm_pkg_find(alpm_list_t *haystack, const char *needle)
 	for(lp = haystack; lp; lp = lp->next) {
 		pmpkg_t *info = lp->data;
 
-		if(info && strcmp(alpm_pkg_get_name(info), needle) == 0) {
+		if(info && strcmp(info->name, needle) == 0) {
 			return(info);
 		}
 	}
