@@ -202,7 +202,7 @@ int SYMEXPORT alpm_sync_sysupgrade(int enable_downgrade)
 	return(0);
 }
 
-int _alpm_sync_pkg(pmpkg_t *spkg)
+static int sync_pkg(pmpkg_t *spkg, alpm_list_t *pkg_list)
 {
 	pmtrans_t *trans;
 	pmdb_t *db_local;
@@ -213,7 +213,7 @@ int _alpm_sync_pkg(pmpkg_t *spkg)
 	trans = handle->trans;
 	db_local = handle->db_local;
 
-	if(_alpm_pkg_find(trans->add, alpm_pkg_get_name(spkg))) {
+	if(_alpm_pkg_find(pkg_list, alpm_pkg_get_name(spkg))) {
 		RET_ERR(PM_ERR_TRANS_DUP_TARGET, -1);
 	}
 
@@ -248,9 +248,10 @@ int _alpm_sync_pkg(pmpkg_t *spkg)
 	return(0);
 }
 
-int _alpm_sync_target(alpm_list_t *dbs_sync, char *target)
+static int sync_target(alpm_list_t *dbs_sync, char *target)
 {
 	alpm_list_t *i, *j;
+	alpm_list_t *known_pkgs = NULL;
 	pmpkg_t *spkg;
 	pmdepend_t *dep; /* provisions and dependencies are also allowed */
 	pmgrp_t *grp;
@@ -267,7 +268,7 @@ int _alpm_sync_target(alpm_list_t *dbs_sync, char *target)
 	_alpm_dep_free(dep);
 
 	if(spkg != NULL) {
-		return(_alpm_sync_pkg(spkg));
+		return(sync_pkg(spkg, handle->trans->add));
 	}
 
 	_alpm_log(PM_LOG_DEBUG, "%s package not found, searching for group...\n", target);
@@ -278,20 +279,27 @@ int _alpm_sync_target(alpm_list_t *dbs_sync, char *target)
 			found = 1;
 			for(j = alpm_grp_get_pkgs(grp); j; j = j->next) {
 				pmpkg_t *pkg = j->data;
-				if(_alpm_sync_pkg(pkg) == -1) {
+				if(sync_pkg(pkg, known_pkgs) == -1) {
 					if(pm_errno == PM_ERR_TRANS_DUP_TARGET || pm_errno == PM_ERR_PKG_IGNORED) {
 						/* just skip duplicate or ignored targets */
 						continue;
 					} else {
+						alpm_list_free(known_pkgs);
 						return(-1);
 					}
 				}
+				known_pkgs = alpm_list_add(known_pkgs, pkg);
 			}
 		}
 	}
+	alpm_list_free(known_pkgs);
 
 	if(!found) {
-		RET_ERR(PM_ERR_PKG_NOT_FOUND, -1);
+		/* pass through any 'found but ignored' errors */
+		if(pm_errno != PM_ERR_PKG_IGNORED) {
+			pm_errno = PM_ERR_PKG_NOT_FOUND;
+		}
+		return(-1);
 	}
 
 	return(0);
@@ -325,7 +333,7 @@ int SYMEXPORT alpm_sync_dbtarget(char *dbname, char *target)
 	if(dbs == NULL) {
 		RET_ERR(PM_ERR_PKG_REPO_NOT_FOUND, -1);
 	}
-	return(_alpm_sync_target(dbs, target));
+	return(sync_target(dbs, target));
 }
 
 /** Add a sync target to the transaction.
@@ -342,7 +350,7 @@ int SYMEXPORT alpm_sync_target(char *target)
 	ASSERT(handle != NULL, RET_ERR(PM_ERR_HANDLE_NULL, -1));
 	dbs_sync = handle->dbs_sync;
 
-	return(_alpm_sync_target(dbs_sync,target));
+	return(sync_target(dbs_sync,target));
 }
 
 /** Compute the size of the files that will be downloaded to install a
