@@ -1,7 +1,7 @@
 /*
  *  callback.c
  *
- *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2010 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -90,20 +90,20 @@ static float get_update_timediff(int first_call)
 static void fill_progress(const int bar_percent, const int disp_percent,
 		const int proglen)
 {
-	const unsigned int hashlen = proglen - 8;
-	const unsigned int hash = bar_percent * hashlen / 100;
-	static unsigned int lasthash = 0, mouth = 0;
-	unsigned int i;
+	/* 8 = 1 space + 1 [ + 1 ] + 5 for percent */
+	const int hashlen = proglen - 8;
+	const int hash = bar_percent * hashlen / 100;
+	static int lasthash = 0, mouth = 0;
+	int i;
 
 	if(bar_percent == 0) {
 		lasthash = 0;
 		mouth = 0;
 	}
 
-	/* magic numbers, how I loathe thee */
-	if(proglen > 8) {
+	if(hashlen > 0) {
 		printf(" [");
-		for(i = hashlen; i > 1; --i) {
+		for(i = hashlen; i > 0; --i) {
 			/* if special progress bar enabled */
 			if(config->chomp) {
 				if(i > hashlen - hash) {
@@ -139,7 +139,8 @@ static void fill_progress(const int bar_percent, const int disp_percent,
 		printf("]");
 	}
 	/* print display percent after progress bar */
-	if(proglen > 5) {
+	/* 5 = 1 space + 3 digits + 1 % */
+	if(proglen >= 5) {
 		printf(" %3d%%", disp_percent);
 	}
 
@@ -259,10 +260,20 @@ void cb_trans_conv(pmtransconv_t event, void *data1, void *data2,
 					alpm_pkg_get_name(data2));
 			break;
 		case PM_TRANS_CONV_CONFLICT_PKG:
-			*response = yesno(_(":: %s conflicts with %s. Remove %s?"),
-					(char *)data1,
-					(char *)data2,
-					(char *)data2);
+			/* data parameters: target package, local package, conflict (strings) */
+			/* print conflict only if it contains new information */
+			if(!strcmp(data1, data3) || !strcmp(data2, data3)) {
+				*response = noyes(_(":: %s and %s are in conflict. Remove %s?"),
+						(char *)data1,
+						(char *)data2,
+						(char *)data2);
+			} else {
+				*response = noyes(_(":: %s and %s are in conflict (%s). Remove %s?"),
+						(char *)data1,
+						(char *)data2,
+						(char *)data3,
+						(char *)data2);
+			}
 			break;
 		case PM_TRANS_CONV_REMOVE_PKGS:
 			{
@@ -275,7 +286,7 @@ void cb_trans_conv(pmtransconv_t event, void *data1, void *data2,
 				printf(_(":: the following package(s) cannot be upgraded due to "
 						"unresolvable dependencies:\n"));
 				list_display("     ", namelist);
-				*response = yesno(_("\nDo you want to skip the above "
+				*response = noyes(_("\nDo you want to skip the above "
 							"package(s) for this upgrade?"));
 				alpm_list_free(namelist);
 			}
@@ -294,6 +305,12 @@ void cb_trans_conv(pmtransconv_t event, void *data1, void *data2,
 					(char *)data1);
 			break;
 	}
+	if(config->noask) {
+		if(config->ask & event) {
+			/* inverse the default answer */
+			*response = !*response;
+		}
+	}
 }
 
 /* callback to handle display of transaction progress */
@@ -303,7 +320,7 @@ void cb_trans_progress(pmtransprog_t event, const char *pkgname, int percent,
 	float timediff;
 
 	/* size of line to allocate for text printing (e.g. not progressbar) */
-	const int infolen = 50;
+	int infolen;
 	int tmp, digits, textlen;
 	char *opr = NULL;
 	/* used for wide character width determination and printing */
@@ -314,6 +331,11 @@ void cb_trans_progress(pmtransprog_t event, const char *pkgname, int percent,
 		return;
 	}
 
+	infolen = getcols() * 6 / 10;
+	if (infolen < 50) {
+		infolen = 50;
+	}
+
 	if(percent == 0) {
 		timediff = get_update_timediff(1);
 	} else {
@@ -349,6 +371,8 @@ void cb_trans_progress(pmtransprog_t event, const char *pkgname, int percent,
 		case PM_TRANS_PROGRESS_CONFLICTS_START:
 			opr = _("checking for file conflicts");
 			break;
+		default:
+			return;
 	}
 
 	/* find # of digits in package counts to scale output */
@@ -358,7 +382,7 @@ void cb_trans_progress(pmtransprog_t event, const char *pkgname, int percent,
 		++digits;
 	}
 	/* determine room left for non-digits text [not ( 1/12) part] */
-	textlen = infolen - 3 - (2 * digits);
+	textlen = infolen - 3 /* (/) */ - (2 * digits) - 1 /* space */;
 
 	/* In order to deal with characters from all locales, we have to worry
 	 * about wide characters and their column widths. A lot of stuff is
@@ -432,8 +456,8 @@ void cb_dl_total(off_t total)
 /* callback to handle display of download progress */
 void cb_dl_progress(const char *filename, off_t file_xfered, off_t file_total)
 {
-	const int infolen = 50;
-	const int filenamelen = infolen - 27;
+	int infolen;
+	int filenamelen;
 	char *fname, *p;
 	/* used for wide character width determination and printing */
 	int len, wclen, wcwid, padwid;
@@ -454,6 +478,13 @@ void cb_dl_progress(const char *filename, off_t file_xfered, off_t file_total)
 		return;
 	}
 
+	infolen = getcols() * 6 / 10;
+	if (infolen < 50) {
+		infolen = 50;
+	}
+	/* explanation of magic 28 number at the end */
+	filenamelen = infolen - 28;
+
 	/* only use TotalDownload if enabled and we have a callback value */
 	if(config->totaldownload && list_total) {
 		/* sanity check */
@@ -488,7 +519,7 @@ void cb_dl_progress(const char *filename, off_t file_xfered, off_t file_total)
 			gettimeofday(&initial_time, NULL);
 			xfered_last = (off_t)0;
 			rate_last = 0.0;
-			timediff = get_update_timediff(1);
+			get_update_timediff(1);
 		}
 	} else if(file_xfered == file_total) {
 		/* compute final values */
@@ -539,7 +570,7 @@ void cb_dl_progress(const char *filename, off_t file_xfered, off_t file_total)
 
 	fname = strdup(filename);
 	/* strip package or DB extension for cleaner look */
-	if((p = strstr(fname, PKGEXT)) || (p = strstr(fname, DBEXT))) {
+	if((p = strstr(fname, ".pkg.tar.")) || (p = strstr(fname, ".db.tar."))) {
 			*p = '\0';
 	}
 	/* In order to deal with characters from all locales, we have to worry
@@ -595,6 +626,7 @@ void cb_dl_progress(const char *filename, off_t file_xfered, off_t file_total)
 		}
 	}
 
+	/* 1 space + filenamelen + 1 space + 7 for size + 1 + 7 for rate + 2 for /s + 1 space + 8 for eta */
 	printf(" %ls%-*s %6.1f%c %#6.1f%c/s %02u:%02u:%02u", wcfname,
 			padwid, "", f_xfered, xfered_size,
 			rate, rate_size, eta_h, eta_m, eta_s);
