@@ -1,7 +1,7 @@
 /*
  *  pacman.c
  *
- *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2010 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -34,10 +34,12 @@
 #include <signal.h>
 #include <unistd.h>
 #include <sys/types.h>
+#include <sys/stat.h>
 #include <sys/utsname.h> /* uname */
 #include <locale.h> /* setlocale */
 #include <time.h> /* time_t */
 #include <errno.h>
+#include <glob.h>
 #if defined(PACMAN_DEBUG) && defined(HAVE_MCHECK_H)
 #include <mcheck.h> /* debug tracing (mtrace) */
 #endif
@@ -76,10 +78,11 @@ static void usage(int op, const char * const myname)
 		printf(_("operations:\n"));
 		printf("    %s {-h --help}\n", myname);
 		printf("    %s {-V --version}\n", myname);
-		printf("    %s {-Q --query}   [%s] [%s]\n", myname, str_opt, str_pkg);
-		printf("    %s {-R --remove}  [%s] <%s>\n", myname, str_opt, str_pkg);
-		printf("    %s {-S --sync}    [%s] [%s]\n", myname, str_opt, str_pkg);
-		printf("    %s {-U --upgrade} [%s] <%s>\n", myname, str_opt, str_file);
+		printf("    %s {-D --database} <%s> <%s>\n", myname, str_opt, str_pkg);
+		printf("    %s {-Q --query}    [%s] [%s]\n", myname, str_opt, str_pkg);
+		printf("    %s {-R --remove}   [%s] <%s>\n", myname, str_opt, str_pkg);
+		printf("    %s {-S --sync}     [%s] [%s]\n", myname, str_opt, str_pkg);
+		printf("    %s {-U --upgrade}  [%s] <%s>\n", myname, str_opt, str_file);
 		printf(_("\nuse '%s {-h --help}' with an operation for available options\n"),
 				myname);
 	} else {
@@ -88,11 +91,14 @@ static void usage(int op, const char * const myname)
 			printf("%s:\n", str_opt);
 			printf(_("  -c, --cascade        remove packages and all packages that depend on them\n"));
 			printf(_("  -d, --nodeps         skip dependency checks\n"));
-			printf(_("  -k, --dbonly         only remove database entry, do not remove files\n"));
+			printf(_("  -k, --dbonly         only remove database entries, do not remove files\n"));
 			printf(_("  -n, --nosave         remove configuration files as well\n"));
 			printf(_("  -s, --recursive      remove dependencies also (that won't break packages)\n"
 				 "                       (-ss includes explicitly installed dependencies too)\n"));
 			printf(_("  -u, --unneeded       remove unneeded packages (that won't break packages)\n"));
+			printf(_("      --print          only print the targets instead of performing the operation\n"));
+			printf(_("      --print-format <string>\n"
+			         "                       specify how the targets should be printed\n"));
 		} else if(op == PM_OP_UPGRADE) {
 			printf("%s:  %s {-U --upgrade} [%s] <%s>\n", str_usg, myname, str_opt, str_file);
 			printf("%s:\n", str_opt);
@@ -100,6 +106,10 @@ static void usage(int op, const char * const myname)
 			printf(_("      --asexplicit     install packages as explicitly installed\n"));
 			printf(_("  -d, --nodeps         skip dependency checks\n"));
 			printf(_("  -f, --force          force install, overwrite conflicting files\n"));
+			printf(_("  -k, --dbonly         add database entries, do not install or keep existing files\n"));
+			printf(_("      --print          only print the targets instead of performing the operation\n"));
+			printf(_("      --print-format <string>\n"
+			         "                       specify how the targets should be printed\n"));
 		} else if(op == PM_OP_QUERY) {
 			printf("%s:  %s {-Q --query} [%s] [%s]\n", str_usg, myname, str_opt, str_pkg);
 			printf("%s:\n", str_opt);
@@ -128,7 +138,6 @@ static void usage(int op, const char * const myname)
 			printf(_("  -g, --groups         view all members of a package group\n"));
 			printf(_("  -i, --info           view package information\n"));
 			printf(_("  -l, --list <repo>    view a list of packages in a repo\n"));
-			printf(_("  -p, --print-uris     print out URIs for given packages and their dependencies\n"));
 			printf(_("  -s, --search <regex> search remote repositories for matching strings\n"));
 			printf(_("  -u, --sysupgrade     upgrade installed packages (-uu allows downgrade)\n"));
 			printf(_("  -w, --downloadonly   download packages but do not install/upgrade anything\n"));
@@ -137,7 +146,15 @@ static void usage(int op, const char * const myname)
 			printf(_("      --ignore <pkg>   ignore a package upgrade (can be used more than once)\n"));
 			printf(_("      --ignoregroup <grp>\n"
 			         "                       ignore a group upgrade (can be used more than once)\n"));
+			printf(_("      --print          only print the targets instead of performing the operation\n"));
+			printf(_("      --print-format <string>\n"
+			         "                       specify how the targets should be printed\n"));
 			printf(_("  -q, --quiet          show less information for query and search\n"));
+		} else if (op == PM_OP_DATABASE) {
+			printf("%s:  %s {-D --database} <%s> <%s>\n", str_usg, myname, str_opt, str_pkg);
+			printf("%s:\n", str_opt);
+			printf(_("      --asdeps         mark packages as non-explicitly installed\n"));
+			printf(_("      --asexplicit     mark packages as explicitly installed\n"));
 		}
 		printf(_("      --config <path>  set an alternate configuration file\n"));
 		printf(_("      --logfile <path> set an alternate log file\n"));
@@ -149,6 +166,7 @@ static void usage(int op, const char * const myname)
 		printf(_("  -r, --root <path>    set an alternate installation root\n"));
 		printf(_("  -b, --dbpath <path>  set an alternate database location\n"));
 		printf(_("      --cachedir <dir> set an alternate package cache location\n"));
+		printf(_("      --arch <arch>    set an alternate architecture\n"));
 	}
 }
 
@@ -158,7 +176,7 @@ static void version(void)
 {
 	printf("\n");
 	printf(" .--.                  Pacman v%s - libalpm v%s\n", PACKAGE_VERSION, alpm_version());
-	printf("/ _.-' .-.  .-.  .-.   Copyright (C) 2006-2009 Pacman Development Team\n");
+	printf("/ _.-' .-.  .-.  .-.   Copyright (C) 2006-2010 Pacman Development Team\n");
 	printf("\\  '-. '-'  '-'  '-'   Copyright (C) 2002-2006 Judd Vinet\n");
 	printf(" '--'\n");
 	printf(_("                       This program may be freely redistributed under\n"
@@ -195,6 +213,19 @@ static void setuseragent(void)
 	setenv("HTTP_USER_AGENT", agent, 0);
 }
 
+static void setarch(const char *arch)
+{
+	if (strcmp(arch, "auto") == 0) {
+		struct utsname un;
+		uname(&un);
+		pm_printf(PM_LOG_DEBUG, "config: architecture: %s\n", un.machine);
+		alpm_option_set_arch(un.machine);
+	} else {
+		pm_printf(PM_LOG_DEBUG, "config: architecture: %s\n", arch);
+		alpm_option_set_arch(arch);
+	}
+}
+
 /** Free the resources.
  *
  * @param ret the return value
@@ -202,7 +233,7 @@ static void setuseragent(void)
 static void cleanup(int ret) {
 	/* free alpm library resources */
 	if(alpm_release() == -1) {
-		pm_printf(PM_LOG_ERROR, alpm_strerrorlast());
+		pm_printf(PM_LOG_ERROR, "%s\n", alpm_strerrorlast());
 	}
 
 	/* free memory */
@@ -220,7 +251,9 @@ static void cleanup(int ret) {
 static ssize_t xwrite(int fd, const void *buf, size_t count)
 {
 	ssize_t ret;
-	while((ret = write(fd, buf, count)) == -1 && errno == EINTR);
+	do {
+		ret = write(fd, buf, count);
+	} while(ret == -1 && errno == EINTR);
 	return(ret);
 }
 
@@ -317,6 +350,8 @@ static void setlibpaths(void)
 	}
 }
 
+#define check_optarg() if(!optarg) { return(1); }
+
 /** Parse command-line arguments for each operation.
  * @param argc argc
  * @param argv argv
@@ -328,6 +363,7 @@ static int parseargs(int argc, char *argv[])
 	int option_index = 0;
 	static struct option opts[] =
 	{
+		{"database",   no_argument,       0, 'D'},
 		{"query",      no_argument,       0, 'Q'},
 		{"remove",     no_argument,       0, 'R'},
 		{"sync",       no_argument,       0, 'S'},
@@ -352,7 +388,7 @@ static int parseargs(int argc, char *argv[])
 		{"nosave",     no_argument,       0, 'n'},
 		{"owns",       no_argument,       0, 'o'},
 		{"file",       no_argument,       0, 'p'},
-		{"print-uris", no_argument,       0, 'p'},
+		{"print",      no_argument,       0, 'p'},
 		{"quiet",      no_argument,       0, 'q'},
 		{"root",       required_argument, 0, 'r'},
 		{"recursive",  no_argument,       0, 's'},
@@ -364,22 +400,25 @@ static int parseargs(int argc, char *argv[])
 		{"verbose",    no_argument,       0, 'v'},
 		{"downloadonly", no_argument,     0, 'w'},
 		{"refresh",    no_argument,       0, 'y'},
-		{"noconfirm",  no_argument,       0, 1000},
-		{"config",     required_argument, 0, 1001},
-		{"ignore",     required_argument, 0, 1002},
-		{"debug",      optional_argument, 0, 1003},
-		{"noprogressbar", no_argument,    0, 1004},
-		{"noscriptlet", no_argument,      0, 1005},
-		{"cachedir",   required_argument, 0, 1007},
-		{"asdeps",     no_argument,       0, 1008},
-		{"logfile",    required_argument, 0, 1009},
-		{"ignoregroup", required_argument, 0, 1010},
-		{"needed",     no_argument,       0, 1011},
-		{"asexplicit",     no_argument,   0, 1012},
+		{"noconfirm",  no_argument,       0, OP_NOCONFIRM},
+		{"config",     required_argument, 0, OP_CONFIG},
+		{"ignore",     required_argument, 0, OP_IGNORE},
+		{"debug",      optional_argument, 0, OP_DEBUG},
+		{"noprogressbar", no_argument,    0, OP_NOPROGRESSBAR},
+		{"noscriptlet", no_argument,      0, OP_NOSCRIPTLET},
+		{"ask",        required_argument, 0, OP_ASK},
+		{"cachedir",   required_argument, 0, OP_CACHEDIR},
+		{"asdeps",     no_argument,       0, OP_ASDEPS},
+		{"logfile",    required_argument, 0, OP_LOGFILE},
+		{"ignoregroup", required_argument, 0, OP_IGNOREGROUP},
+		{"needed",     no_argument,       0, OP_NEEDED},
+		{"asexplicit",     no_argument,   0, OP_ASEXPLICIT},
+		{"arch",       required_argument, 0, OP_ARCH},
+		{"print-format", required_argument, 0, OP_PRINTFORMAT},
 		{0, 0, 0, 0}
 	};
 
-	while((opt = getopt_long(argc, argv, "RUQSTr:b:vkhscVfmnoldepqituwygz", opts, &option_index))) {
+	while((opt = getopt_long(argc, argv, "DQRSTUVb:cdefghiklmnopqr:stuvwy", opts, &option_index))) {
 		alpm_list_t *list = NULL, *item = NULL; /* lists for splitting strings */
 
 		if(opt < 0) {
@@ -387,21 +426,23 @@ static int parseargs(int argc, char *argv[])
 		}
 		switch(opt) {
 			case 0: break;
-			case 1000: config->noconfirm = 1; break;
-			case 1001:
+			case OP_NOCONFIRM: config->noconfirm = 1; break;
+			case OP_CONFIG:
+				check_optarg();
 				if(config->configfile) {
 					free(config->configfile);
 				}
 				config->configfile = strndup(optarg, PATH_MAX);
 				break;
-			case 1002:
+			case OP_IGNORE:
+				check_optarg();
 				list = strsplit(optarg, ',');
 				for(item = list; item; item = alpm_list_next(item)) {
 					alpm_option_add_ignorepkg((char *)alpm_list_getdata(item));
 				}
 				FREELIST(list);
 				break;
-			case 1003:
+			case OP_DEBUG:
 				/* debug levels are made more 'human readable' than using a raw logmask
 				 * here, error and warning are set in config_new, though perhaps a
 				 * --quiet option will remove these later */
@@ -424,32 +465,49 @@ static int parseargs(int argc, char *argv[])
 				/* progress bars get wonky with debug on, shut them off */
 				config->noprogressbar = 1;
 				break;
-			case 1004: config->noprogressbar = 1; break;
-			case 1005: config->flags |= PM_TRANS_FLAG_NOSCRIPTLET; break;
-			case 1007:
+			case OP_NOPROGRESSBAR: config->noprogressbar = 1; break;
+			case OP_NOSCRIPTLET: config->flags |= PM_TRANS_FLAG_NOSCRIPTLET; break;
+			case OP_ASK:
+				check_optarg();
+				config->noask = 1;
+				config->ask = atoi(optarg);
+				break;
+			case OP_CACHEDIR:
+				check_optarg();
 				if(alpm_option_add_cachedir(optarg) != 0) {
 					pm_printf(PM_LOG_ERROR, _("problem adding cachedir '%s' (%s)\n"),
 							optarg, alpm_strerrorlast());
 					return(1);
 				}
 				break;
-			case 1008:
+			case OP_ASDEPS:
 				config->flags |= PM_TRANS_FLAG_ALLDEPS;
 				break;
-			case 1009:
+			case OP_LOGFILE:
+				check_optarg();
 				config->logfile = strndup(optarg, PATH_MAX);
 				break;
-			case 1010:
+			case OP_IGNOREGROUP:
+				check_optarg();
 				list = strsplit(optarg, ',');
 				for(item = list; item; item = alpm_list_next(item)) {
 					alpm_option_add_ignoregrp((char *)alpm_list_getdata(item));
 				}
 				FREELIST(list);
 				break;
-			case 1011: config->flags |= PM_TRANS_FLAG_NEEDED; break;
-			case 1012:
+			case OP_NEEDED: config->flags |= PM_TRANS_FLAG_NEEDED; break;
+			case OP_ASEXPLICIT:
 				config->flags |= PM_TRANS_FLAG_ALLEXPLICIT;
 				break;
+			case OP_ARCH:
+				check_optarg();
+				setarch(optarg);
+				break;
+			case OP_PRINTFORMAT:
+				check_optarg();
+				config->print_format = strdup(optarg);
+				break;
+			case 'D': config->op = (config->op != PM_OP_MAIN ? 0 : PM_OP_DATABASE); break;
 			case 'Q': config->op = (config->op != PM_OP_MAIN ? 0 : PM_OP_QUERY); break;
 			case 'R': config->op = (config->op != PM_OP_MAIN ? 0 : PM_OP_REMOVE); break;
 			case 'S': config->op = (config->op != PM_OP_MAIN ? 0 : PM_OP_SYNC); break;
@@ -457,6 +515,7 @@ static int parseargs(int argc, char *argv[])
 			case 'U': config->op = (config->op != PM_OP_MAIN ? 0 : PM_OP_UPGRADE); break;
 			case 'V': config->version = 1; break;
 			case 'b':
+				check_optarg();
 				config->dbpath = strdup(optarg);
 				break;
 			case 'c':
@@ -485,14 +544,13 @@ static int parseargs(int argc, char *argv[])
 			case 'o': config->op_q_owns = 1; break;
 			case 'p':
 				config->op_q_isfile = 1;
-				config->op_s_printuris = 1;
-				config->flags |= PM_TRANS_FLAG_NOCONFLICTS;
-				config->flags |= PM_TRANS_FLAG_NOLOCK;
+				config->print = 1;
 				break;
 			case 'q':
 				config->quiet = 1;
 				break;
 			case 'r':
+				check_optarg();
 				config->rootdir = strdup(optarg);
 				break;
 			case 's':
@@ -592,7 +650,7 @@ static char *get_filename(const char *url) {
 static char *get_destfile(const char *path, const char *filename) {
 	char *destfile;
 	/* len = localpath len + filename len + null */
-	int len = strlen(path) + strlen(filename) + 1;
+	size_t len = strlen(path) + strlen(filename) + 1;
 	destfile = calloc(len, sizeof(char));
 	snprintf(destfile, len, "%s%s", path, filename);
 
@@ -602,7 +660,7 @@ static char *get_destfile(const char *path, const char *filename) {
 static char *get_tempfile(const char *path, const char *filename) {
 	char *tempfile;
 	/* len = localpath len + filename len + '.part' len + null */
-	int len = strlen(path) + strlen(filename) + 6;
+	size_t len = strlen(path) + strlen(filename) + 6;
 	tempfile = calloc(len, sizeof(char));
 	snprintf(tempfile, len, "%s%s.part", path, filename);
 
@@ -611,13 +669,12 @@ static char *get_tempfile(const char *path, const char *filename) {
 
 /** External fetch callback */
 int download_with_xfercommand(const char *url, const char *localpath,
-		time_t mtimeold, time_t *mtimenew) {
+		int force) {
 	int ret = 0;
 	int retval;
 	int usepart = 0;
-	char *ptr1, *ptr2;
-	char origCmd[PATH_MAX];
-	char parsedCmd[PATH_MAX] = "";
+	struct stat st;
+	char *parsedcmd,*tempcmd;
 	char cwd[PATH_MAX];
 	char *destfile, *tempfile, *filename;
 
@@ -632,28 +689,25 @@ int download_with_xfercommand(const char *url, const char *localpath,
 	destfile = get_destfile(localpath, filename);
 	tempfile = get_tempfile(localpath, filename);
 
-	strncpy(origCmd, config->xfercommand, sizeof(origCmd));
+	if(force && stat(tempfile, &st) == 0) {
+		unlink(tempfile);
+	}
+	if(force && stat(destfile, &st) == 0) {
+		unlink(destfile);
+	}
+
+	tempcmd = strdup(config->xfercommand);
 	/* replace all occurrences of %o with fn.part */
-	ptr1 = origCmd;
-	while((ptr2 = strstr(ptr1, "%o"))) {
+	if(strstr(tempcmd, "%o")) {
 		usepart = 1;
-		ptr2[0] = '\0';
-		strcat(parsedCmd, ptr1);
-		strcat(parsedCmd, tempfile);
-		ptr1 = ptr2 + 2;
+		parsedcmd = strreplace(tempcmd, "%o", tempfile);
+		free(tempcmd);
+		tempcmd = parsedcmd;
 	}
-	strcat(parsedCmd, ptr1);
 	/* replace all occurrences of %u with the download URL */
-	strncpy(origCmd, parsedCmd, sizeof(origCmd));
-	parsedCmd[0] = '\0';
-	ptr1 = origCmd;
-	while((ptr2 = strstr(ptr1, "%u"))) {
-		ptr2[0] = '\0';
-		strcat(parsedCmd, ptr1);
-		strcat(parsedCmd, url);
-		ptr1 = ptr2 + 2;
-	}
-	strcat(parsedCmd, ptr1);
+	parsedcmd = strreplace(tempcmd, "%u", url);
+	free(tempcmd);
+
 	/* cwd to the download directory */
 	getcwd(cwd, PATH_MAX);
 	if(chdir(localpath)) {
@@ -662,8 +716,8 @@ int download_with_xfercommand(const char *url, const char *localpath,
 		goto cleanup;
 	}
 	/* execute the parsed command via /bin/sh -c */
-	pm_printf(PM_LOG_DEBUG, "running command: %s\n", parsedCmd);
-	retval = system(parsedCmd);
+	pm_printf(PM_LOG_DEBUG, "running command: %s\n", parsedcmd);
+	retval = system(parsedcmd);
 
 	if(retval == -1) {
 		pm_printf(PM_LOG_WARNING, _("running XferCommand: fork failed!\n"));
@@ -689,10 +743,132 @@ cleanup:
 	}
 	free(destfile);
 	free(tempfile);
+	free(parsedcmd);
 
 	return(ret);
 }
 
+static int _parse_options(char *key, char *value)
+{
+	if(value == NULL) {
+		/* options without settings */
+		if(strcmp(key, "UseSyslog") == 0) {
+			alpm_option_set_usesyslog(1);
+			pm_printf(PM_LOG_DEBUG, "config: usesyslog\n");
+		} else if(strcmp(key, "ILoveCandy") == 0) {
+			config->chomp = 1;
+			pm_printf(PM_LOG_DEBUG, "config: chomp\n");
+		} else if(strcmp(key, "ShowSize") == 0) {
+			config->showsize = 1;
+			pm_printf(PM_LOG_DEBUG, "config: showsize\n");
+		} else if(strcmp(key, "UseDelta") == 0) {
+			alpm_option_set_usedelta(1);
+			pm_printf(PM_LOG_DEBUG, "config: usedelta\n");
+		} else if(strcmp(key, "TotalDownload") == 0) {
+			config->totaldownload = 1;
+			pm_printf(PM_LOG_DEBUG, "config: totaldownload\n");
+		} else {
+			pm_printf(PM_LOG_ERROR, _("directive '%s' without value not recognized\n"), key);
+			return(1);
+		}
+	} else {
+		/* options with settings */
+		if(strcmp(key, "NoUpgrade") == 0) {
+			setrepeatingoption(value, "NoUpgrade", alpm_option_add_noupgrade);
+		} else if(strcmp(key, "NoExtract") == 0) {
+			setrepeatingoption(value, "NoExtract", alpm_option_add_noextract);
+		} else if(strcmp(key, "IgnorePkg") == 0) {
+			setrepeatingoption(value, "IgnorePkg", alpm_option_add_ignorepkg);
+		} else if(strcmp(key, "IgnoreGroup") == 0) {
+			setrepeatingoption(value, "IgnoreGroup", alpm_option_add_ignoregrp);
+		} else if(strcmp(key, "HoldPkg") == 0) {
+			setrepeatingoption(value, "HoldPkg", option_add_holdpkg);
+		} else if(strcmp(key, "SyncFirst") == 0) {
+			setrepeatingoption(value, "SyncFirst", option_add_syncfirst);
+		} else if(strcmp(key, "Architecture") == 0) {
+			if(!alpm_option_get_arch()) {
+				setarch(value);
+			}
+		} else if(strcmp(key, "DBPath") == 0) {
+			/* don't overwrite a path specified on the command line */
+			if(!config->dbpath) {
+				config->dbpath = strdup(value);
+				pm_printf(PM_LOG_DEBUG, "config: dbpath: %s\n", value);
+			}
+		} else if(strcmp(key, "CacheDir") == 0) {
+			if(alpm_option_add_cachedir(value) != 0) {
+				pm_printf(PM_LOG_ERROR, _("problem adding cachedir '%s' (%s)\n"),
+						value, alpm_strerrorlast());
+				return(1);
+			}
+			pm_printf(PM_LOG_DEBUG, "config: cachedir: %s\n", value);
+		} else if(strcmp(key, "RootDir") == 0) {
+			/* don't overwrite a path specified on the command line */
+			if(!config->rootdir) {
+				config->rootdir = strdup(value);
+				pm_printf(PM_LOG_DEBUG, "config: rootdir: %s\n", value);
+			}
+		} else if (strcmp(key, "LogFile") == 0) {
+			if(!config->logfile) {
+				config->logfile = strdup(value);
+				pm_printf(PM_LOG_DEBUG, "config: logfile: %s\n", value);
+			}
+		} else if (strcmp(key, "XferCommand") == 0) {
+			config->xfercommand = strdup(value);
+			alpm_option_set_fetchcb(download_with_xfercommand);
+			pm_printf(PM_LOG_DEBUG, "config: xfercommand: %s\n", value);
+		} else if (strcmp(key, "CleanMethod") == 0) {
+			if (strcmp(value, "KeepInstalled") == 0) {
+				config->cleanmethod = PM_CLEAN_KEEPINST;
+			} else if (strcmp(value, "KeepCurrent") == 0) {
+				config->cleanmethod = PM_CLEAN_KEEPCUR;
+			} else {
+				pm_printf(PM_LOG_ERROR, _("invalid value for 'CleanMethod' : '%s'\n"), value);
+				return(1);
+			}
+			pm_printf(PM_LOG_DEBUG, "config: cleanmethod: %s\n", value);
+		} else {
+			pm_printf(PM_LOG_ERROR, _("directive '%s' with a value not recognized\n"), key);
+			return(1);
+		}
+
+	}
+	return(0);
+}
+
+static int _add_mirror(pmdb_t *db, char *value)
+{
+	const char *dbname = alpm_db_get_name(db);
+	/* let's attempt a replacement for the current repo */
+	char *temp = strreplace(value, "$repo", dbname);
+	/* let's attempt a replacement for the arch */
+	const char *arch = alpm_option_get_arch();
+	char *server;
+	if(arch) {
+		server = strreplace(temp, "$arch", arch);
+		free(temp);
+	} else {
+		if(strstr(temp, "$arch")) {
+			free(temp);
+			pm_printf(PM_LOG_ERROR, _("The mirror '%s' contains the $arch"
+						" variable, but no Architecture is defined.\n"), value);
+			return(1);
+		}
+		server = temp;
+	}
+
+	if(alpm_db_setserver(db, server) != 0) {
+		/* pm_errno is set by alpm_db_setserver */
+		pm_printf(PM_LOG_ERROR, _("could not add server URL to database '%s': %s (%s)\n"),
+				dbname, server, alpm_strerrorlast());
+		free(server);
+		return(1);
+	}
+
+	free(server);
+	return(0);
+}
+
 /* The real parseconfig. Called with a null section argument by the publicly
  * visible parseconfig so we can recall from within ourself on an include */
 static int _parseconfig(const char *file, const char *givensection,
@@ -749,11 +925,8 @@ static int _parseconfig(const char *file, const char *givensection,
 				ret = 1;
 				goto cleanup;
 			}
-			/* if we are not looking at the options section, register a db and also
-			 * ensure we have set all of our library paths as the library is too stupid
-			 * at the moment to do lazy opening of the databases */
+			/* if we are not looking at the options section, register a db */
 			if(strcmp(section, "options") != 0) {
-				setlibpaths();
 				db = alpm_db_register_sync(section);
 				if(db == NULL) {
 					pm_printf(PM_LOG_ERROR, _("could not register '%s' database (%s)\n"),
@@ -762,142 +935,100 @@ static int _parseconfig(const char *file, const char *givensection,
 					goto cleanup;
 				}
 			}
-		} else {
-			/* directive */
-			char *key;
-			/* strsep modifies the 'line' string: 'key \0 ptr' */
-			key = line;
-			ptr = line;
-			strsep(&ptr, "=");
-			strtrim(key);
-			strtrim(ptr);
+			continue;
+		}
 
-			if(key == NULL) {
-				pm_printf(PM_LOG_ERROR, _("config file %s, line %d: syntax error in config file- missing key.\n"),
+		/* directive */
+		char *key, *value;
+		/* strsep modifies the 'line' string: 'key \0 value' */
+		key = line;
+		value = line;
+		strsep(&value, "=");
+		strtrim(key);
+		strtrim(value);
+
+		if(key == NULL) {
+			pm_printf(PM_LOG_ERROR, _("config file %s, line %d: syntax error in config file- missing key.\n"),
+					file, linenum);
+			ret = 1;
+			goto cleanup;
+		}
+		/* For each directive, compare to the camelcase string. */
+		if(section == NULL) {
+			pm_printf(PM_LOG_ERROR, _("config file %s, line %d: All directives must belong to a section.\n"),
+					file, linenum);
+			ret = 1;
+			goto cleanup;
+		}
+		/* Include is allowed in both options and repo sections */
+		if(strcmp(key, "Include") == 0) {
+			if(value == NULL) {
+				pm_printf(PM_LOG_ERROR, _("config file %s, line %d: directive %s needs a value\n"),
+						file, linenum, key);
+				ret = 1;
+				goto cleanup;
+			}
+			/* Ignore include failures... assume non-critical */
+			int globret;
+			glob_t globbuf;
+			globret = glob(value, GLOB_NOCHECK, NULL, &globbuf);
+			switch(globret) {
+				case GLOB_NOSPACE:
+					pm_printf(PM_LOG_DEBUG,
+							"config file %s, line %d: include globing out of space\n",
+							file, linenum);
+				break;
+				case GLOB_ABORTED:
+					pm_printf(PM_LOG_DEBUG,
+							"config file %s, line %d: include globing read error for %s\n",
+							file, linenum, value);
+				break;
+				case GLOB_NOMATCH:
+					pm_printf(PM_LOG_DEBUG,
+							"config file %s, line %d: no include found for %s\n",
+							file, linenum, value);
+				break;
+				default:
+					for(int gindex = 0; gindex < globbuf.gl_pathc; gindex++) {
+						pm_printf(PM_LOG_DEBUG, "config file %s, line %d: including %s\n",
+								file, linenum, globbuf.gl_pathv[gindex]);
+						_parseconfig(globbuf.gl_pathv[gindex], section, db);
+					}
+				break;
+			}
+			globfree(&globbuf);
+			continue;
+		}
+		if(strcmp(section, "options") == 0) {
+			/* we are either in options ... */
+			if((ret = _parse_options(key, value)) != 0) {
+				pm_printf(PM_LOG_ERROR, _("config file %s, line %d: problem in options section\n"),
 						file, linenum);
 				ret = 1;
 				goto cleanup;
 			}
-			/* For each directive, compare to the camelcase string. */
-			if(section == NULL) {
-				pm_printf(PM_LOG_ERROR, _("config file %s, line %d: All directives must belong to a section.\n"),
-						file, linenum);
-				ret = 1;
-				goto cleanup;
-			}
-			if(ptr == NULL && strcmp(section, "options") == 0) {
-				/* directives without settings, all in [options] */
-				if(strcmp(key, "NoPassiveFtp") == 0) {
-					alpm_option_set_nopassiveftp(1);
-					pm_printf(PM_LOG_DEBUG, "config: nopassiveftp\n");
-				} else if(strcmp(key, "UseSyslog") == 0) {
-					alpm_option_set_usesyslog(1);
-					pm_printf(PM_LOG_DEBUG, "config: usesyslog\n");
-				} else if(strcmp(key, "ILoveCandy") == 0) {
-					config->chomp = 1;
-					pm_printf(PM_LOG_DEBUG, "config: chomp\n");
-				} else if(strcmp(key, "ShowSize") == 0) {
-					config->showsize = 1;
-					pm_printf(PM_LOG_DEBUG, "config: showsize\n");
-				} else if(strcmp(key, "UseDelta") == 0) {
-					alpm_option_set_usedelta(1);
-					pm_printf(PM_LOG_DEBUG, "config: usedelta\n");
-				} else if(strcmp(key, "TotalDownload") == 0) {
-					config->totaldownload = 1;
-					pm_printf(PM_LOG_DEBUG, "config: totaldownload\n");
-				} else {
-					pm_printf(PM_LOG_ERROR, _("config file %s, line %d: directive '%s' not recognized.\n"),
+			continue;
+		} else {
+			/* ... or in a repo section */
+			if(strcmp(key, "Server") == 0) {
+				if(value == NULL) {
+					pm_printf(PM_LOG_ERROR, _("config file %s, line %d: directive %s needs a value\n"),
 							file, linenum, key);
 					ret = 1;
 					goto cleanup;
 				}
+				if(_add_mirror(db, value) != 0) {
+					ret = 1;
+					goto cleanup;
+				}
 			} else {
-				/* directives with settings */
-				if(strcmp(key, "Include") == 0) {
-					pm_printf(PM_LOG_DEBUG, "config: including %s\n", ptr);
-					_parseconfig(ptr, section, db);
-					/* Ignore include failures... assume non-critical */
-				} else if(strcmp(section, "options") == 0) {
-					if(strcmp(key, "NoUpgrade") == 0) {
-						setrepeatingoption(ptr, "NoUpgrade", alpm_option_add_noupgrade);
-					} else if(strcmp(key, "NoExtract") == 0) {
-						setrepeatingoption(ptr, "NoExtract", alpm_option_add_noextract);
-					} else if(strcmp(key, "IgnorePkg") == 0) {
-						setrepeatingoption(ptr, "IgnorePkg", alpm_option_add_ignorepkg);
-					} else if(strcmp(key, "IgnoreGroup") == 0) {
-						setrepeatingoption(ptr, "IgnoreGroup", alpm_option_add_ignoregrp);
-					} else if(strcmp(key, "HoldPkg") == 0) {
-						setrepeatingoption(ptr, "HoldPkg", option_add_holdpkg);
-					} else if(strcmp(key, "SyncFirst") == 0) {
-						setrepeatingoption(ptr, "SyncFirst", option_add_syncfirst);
-					} else if(strcmp(key, "DBPath") == 0) {
-						/* don't overwrite a path specified on the command line */
-						if(!config->dbpath) {
-							config->dbpath = strdup(ptr);
-							pm_printf(PM_LOG_DEBUG, "config: dbpath: %s\n", ptr);
-						}
-					} else if(strcmp(key, "CacheDir") == 0) {
-						if(alpm_option_add_cachedir(ptr) != 0) {
-							pm_printf(PM_LOG_ERROR, _("problem adding cachedir '%s' (%s)\n"),
-									ptr, alpm_strerrorlast());
-							ret = 1;
-							goto cleanup;
-						}
-						pm_printf(PM_LOG_DEBUG, "config: cachedir: %s\n", ptr);
-					} else if(strcmp(key, "RootDir") == 0) {
-						/* don't overwrite a path specified on the command line */
-						if(!config->rootdir) {
-							config->rootdir = strdup(ptr);
-							pm_printf(PM_LOG_DEBUG, "config: rootdir: %s\n", ptr);
-						}
-					} else if (strcmp(key, "LogFile") == 0) {
-						if(!config->logfile) {
-							config->logfile = strdup(ptr);
-							pm_printf(PM_LOG_DEBUG, "config: logfile: %s\n", ptr);
-						}
-					} else if (strcmp(key, "XferCommand") == 0) {
-						config->xfercommand = strdup(ptr);
-						alpm_option_set_fetchcb(download_with_xfercommand);
-						pm_printf(PM_LOG_DEBUG, "config: xfercommand: %s\n", ptr);
-					} else if (strcmp(key, "CleanMethod") == 0) {
-						if (strcmp(ptr, "KeepInstalled") == 0) {
-							config->cleanmethod = PM_CLEAN_KEEPINST;
-						} else if (strcmp(ptr, "KeepCurrent") == 0) {
-							config->cleanmethod = PM_CLEAN_KEEPCUR;
-						} else {
-							pm_printf(PM_LOG_ERROR, _("invalid value for 'CleanMethod' : '%s'\n"), ptr);
-							ret = 1;
-							goto cleanup;
-						}
-						pm_printf(PM_LOG_DEBUG, "config: cleanmethod: %s\n", ptr);
-					} else {
-						pm_printf(PM_LOG_ERROR, _("config file %s, line %d: directive '%s' not recognized.\n"),
-								file, linenum, key);
-						ret = 1;
-						goto cleanup;
-					}
-				} else if(strcmp(key, "Server") == 0) {
-					/* let's attempt a replacement for the current repo */
-					char *server = strreplace(ptr, "$repo", section);
-
-					if(alpm_db_setserver(db, server) != 0) {
-						/* pm_errno is set by alpm_db_setserver */
-						pm_printf(PM_LOG_ERROR, _("could not add server URL to database '%s': %s (%s)\n"),
-								alpm_db_get_name(db), server, alpm_strerrorlast());
-						free(server);
-						ret = 1;
-						goto cleanup;
-					}
-
-					free(server);
-				} else {
-					pm_printf(PM_LOG_ERROR, _("config file %s, line %d: directive '%s' not recognized.\n"),
-							file, linenum, key);
-					ret = 1;
-					goto cleanup;
-				}
+				pm_printf(PM_LOG_ERROR, _("config file %s, line %d: directive '%s' in repository section '%s' not recognized.\n"),
+						file, linenum, key, section);
+				ret = 1;
+				goto cleanup;
 			}
 		}
+
 	}
 
 cleanup:
@@ -923,6 +1054,29 @@ static int parseconfig(const char *file)
 	return(_parseconfig(file, NULL, NULL));
 }
 
+/** print commandline to logfile
+ */
+static void cl_to_log(int argc, char* argv[])
+{
+	size_t size = 0;
+	int i;
+	for(i = 0; i<argc; i++) {
+		size += strlen(argv[i]) + 1;
+	}
+	char *cl_text = malloc(size);
+	if(!cl_text)
+		return;
+	char *p = cl_text;
+	for(i = 0; i<argc-1; i++) {
+		strcpy(p, argv[i]);
+		p += strlen(argv[i]);
+		*p++ = ' ';
+	}
+	strcpy(p, argv[i]);
+	alpm_logaction("Running '%s'\n", cl_text);
+	free(cl_text);
+}
+
 /** Main function.
  * @param argc argc
  * @param argv argv
@@ -1018,6 +1172,20 @@ int main(int argc, char *argv[])
 		alpm_option_set_totaldlcb(cb_dl_total);
 	}
 
+	/* noask is meant to be non-interactive */
+	if(config->noask) {
+		config->noconfirm = 1;
+	}
+
+	/* set up the print operations */
+	if(config->print) {
+		config->noconfirm = 1;
+		config->flags |= PM_TRANS_FLAG_NOCONFLICTS;
+		config->flags |= PM_TRANS_FLAG_NOLOCK;
+		/* Display only errors */
+		config->logmask &= ~PM_LOG_WARNING;
+	}
+
 #if defined(HAVE_GETEUID) && !defined(CYGWIN)
 	/* check if we have sufficient permission for the requested operation */
 	if(myuid > 0 && needs_root()) {
@@ -1049,8 +1217,16 @@ int main(int argc, char *argv[])
 		cleanup(EXIT_FAILURE);
 	}
 
+	/* Log commandline */
+	if(needs_root()) {
+		cl_to_log(argc, argv);
+	}
+
 	/* start the requested operation */
 	switch(config->op) {
+		case PM_OP_DATABASE:
+			ret = pacman_database(pm_targets);
+			break;
 		case PM_OP_REMOVE:
 			ret = pacman_remove(pm_targets);
 			break;
