@@ -1,7 +1,7 @@
 /*
  *  pacman.h
  *
- *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2010 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -22,6 +22,8 @@
 
 #include <alpm_list.h>
 
+/* database.c */
+int pacman_database(alpm_list_t *targets);
 /* query.c */
 int pacman_query(alpm_list_t *targets);
 /* remove.c */
