@@ -1,7 +1,7 @@
 /*
  *  upgrade.c
  *
- *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2010 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2006 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -42,7 +42,6 @@
 int pacman_upgrade(alpm_list_t *targets)
 {
 	alpm_list_t *i, *data = NULL;
-	pmtranstype_t transtype = PM_TRANS_TYPE_UPGRADE;
 	int retval = 0;
 
 	if(targets == NULL) {
@@ -65,15 +64,14 @@ int pacman_upgrade(alpm_list_t *targets)
 	}
 
 	/* Step 1: create a new transaction */
-	if(trans_init(transtype, config->flags) == -1) {
+	if(trans_init(config->flags) == -1) {
 		return(1);
 	}
 
 	/* add targets to the created transaction */
-	printf(_("loading package data...\n"));
 	for(i = targets; i; i = alpm_list_next(i)) {
 		char *targ = alpm_list_getdata(i);
-		if(alpm_trans_addtarget(targ) == -1) {
+		if(alpm_add_target(targ) == -1) {
 			pm_fprintf(stderr, PM_LOG_ERROR, "'%s': %s\n",
 					targ, alpm_strerrorlast());
 			trans_release();
@@ -87,6 +85,12 @@ int pacman_upgrade(alpm_list_t *targets)
 		pm_fprintf(stderr, PM_LOG_ERROR, _("failed to prepare transaction (%s)\n"),
 		        alpm_strerrorlast());
 		switch(pm_errno) {
+			case PM_ERR_PKG_INVALID_ARCH:
+				for(i = data; i; i = alpm_list_next(i)) {
+					char *pkg = alpm_list_getdata(i);
+					printf(_(":: package %s does not have a valid architecture\n"), pkg);
+				}
+				break;
 			case PM_ERR_UNSATISFIED_DEPS:
 				for(i = data; i; i = alpm_list_next(i)) {
 					pmdepmissing_t *miss = alpm_list_getdata(i);
@@ -104,28 +108,16 @@ int pacman_upgrade(alpm_list_t *targets)
 			case PM_ERR_CONFLICTING_DEPS:
 				for(i = data; i; i = alpm_list_next(i)) {
 					pmconflict_t *conflict = alpm_list_getdata(i);
-					printf(_(":: %s: conflicts with %s\n"),
-						alpm_conflict_get_package1(conflict), alpm_conflict_get_package2(conflict));
-				}
-				break;
-			case PM_ERR_FILE_CONFLICTS:
-				for(i = data; i; i = alpm_list_next(i)) {
-					pmfileconflict_t *conflict = alpm_list_getdata(i);
-					switch(alpm_fileconflict_get_type(conflict)) {
-						case PM_FILECONFLICT_TARGET:
-							printf(_("%s exists in both '%s' and '%s'\n"),
-							        alpm_fileconflict_get_file(conflict),
-							        alpm_fileconflict_get_target(conflict),
-							        alpm_fileconflict_get_ctarget(conflict));
-						break;
-						case PM_FILECONFLICT_FILESYSTEM:
-							printf(_("%s: %s exists in filesystem\n"),
-							        alpm_fileconflict_get_target(conflict),
-							        alpm_fileconflict_get_file(conflict));
-						break;
+					const char *package1 = alpm_conflict_get_package1(conflict);
+					const char *package2 = alpm_conflict_get_package2(conflict);
+					const char *reason = alpm_conflict_get_reason(conflict);
+					/* only print reason if it contains new information */
+					if(!strcmp(package1, reason) || !strcmp(package2, reason)) {
+						printf(_(":: %s and %s are in conflict\n"), package1, package2);
+					} else {
+						printf(_(":: %s and %s are in conflict (%s)\n"), package1, package2, reason);
 					}
 				}
-				printf(_("\nerrors occurred, no packages were upgraded.\n"));
 				break;
 			default:
 				break;
@@ -134,12 +126,65 @@ int pacman_upgrade(alpm_list_t *targets)
 		FREELIST(data);
 		return(1);
 	}
-	alpm_list_free(data);
 
 	/* Step 3: perform the installation */
-	if(alpm_trans_commit(NULL) == -1) {
+
+	if(config->print) {
+		print_packages(alpm_trans_get_add());
+		trans_release();
+		return(0);
+	}
+
+	/* print targets and ask user confirmation */
+	alpm_list_t *packages = alpm_trans_get_add();
+	if(packages == NULL) { /* we are done */
+		printf(_(" there is nothing to do\n"));
+		trans_release();
+		return(retval);
+	}
+	display_targets(alpm_trans_get_remove(), 0);
+	display_targets(alpm_trans_get_add(), 1);
+	printf("\n");
+	int confirm = yesno(_("Proceed with installation?"));
+	if(!confirm) {
+		trans_release();
+		return(retval);
+	}
+
+	if(alpm_trans_commit(&data) == -1) {
 		pm_fprintf(stderr, PM_LOG_ERROR, _("failed to commit transaction (%s)\n"),
-			alpm_strerrorlast());
+				alpm_strerrorlast());
+		switch(pm_errno) {
+			alpm_list_t *i;
+			case PM_ERR_FILE_CONFLICTS:
+				for(i = data; i; i = alpm_list_next(i)) {
+					pmfileconflict_t *conflict = alpm_list_getdata(i);
+					switch(alpm_fileconflict_get_type(conflict)) {
+						case PM_FILECONFLICT_TARGET:
+							printf(_("%s exists in both '%s' and '%s'\n"),
+									alpm_fileconflict_get_file(conflict),
+									alpm_fileconflict_get_target(conflict),
+									alpm_fileconflict_get_ctarget(conflict));
+							break;
+						case PM_FILECONFLICT_FILESYSTEM:
+							printf(_("%s: %s exists in filesystem\n"),
+									alpm_fileconflict_get_target(conflict),
+									alpm_fileconflict_get_file(conflict));
+							break;
+					}
+				}
+				break;
+			case PM_ERR_PKG_INVALID:
+			case PM_ERR_DLT_INVALID:
+				for(i = data; i; i = alpm_list_next(i)) {
+					char *filename = alpm_list_getdata(i);
+					printf(_("%s is invalid or corrupted\n"), filename);
+				}
+				break;
+			default:
+				break;
+		}
+		FREELIST(data);
 		trans_release();
 		return(1);
 	}
