@@ -1,7 +1,7 @@
 /*
  *  vercmp.c
  *
- *  Copyright (c) 2006-2009 Pacman Development Team <pacman-dev@archlinux.org>
+ *  Copyright (c) 2006-2010 Pacman Development Team <pacman-dev@archlinux.org>
  *  Copyright (c) 2002-2005 by Judd Vinet <jvinet@zeroflux.org>
  *
  *  This program is free software; you can redistribute it and/or modify
@@ -23,12 +23,14 @@
 #include <stdio.h> /* printf */
 #include <string.h> /* strncpy */
 
-#include <alpm.h>
-
 #define BASENAME "vercmp"
 
 #define MAX_LEN 255
 
+/* forward declaration, comes from vercmp.o in libalpm source that is linked in
+ * directly so we don't have any library deps */
+int alpm_pkg_vercmp(const char *a, const char *b);
+
 static void usage()
 {
 	fprintf(stderr, "usage: %s <ver1> <ver2>\n\n", BASENAME);
